@@ -296,7 +296,9 @@ class Identify():
             matches = yara_rules.match(path, externals=externals, fast=True)
             matches.sort(key=lambda x: x.meta.get('score', 0), reverse=True)
             for match in matches:
-                return match.meta['type']
+                ftype = match.meta.get("type", None)
+                if ftype:
+                    return ftype
         except Exception as e:
             self.log.warning(f"Yara file identifier failed with error: {str(e)}")
             matches = []
@@ -394,7 +396,7 @@ def zip_ident(path: str, fallback: str) -> str:
     android_dex = False
 
     for file_name in file_list:
-        if file_name[:8] == "META-INF" and file_name[9:] == "MANIFEST.MF":
+        if file_name.startswith("META-INF/"):
             is_jar = True
         elif file_name == "AndroidManifest.xml":
             android_manifest = True
