@@ -36,6 +36,10 @@ OLE_CLSID_GUIDs = {
 # Assemblyline type to file extension mapping
 type_to_extension = {
     "archive/chm": ".chm",
+    "archive/iso": ".iso",
+    "archive/udf": ".udf",
+    "archive/vhd": ".vhd",
+    "archive/zip": ".zip",
     "audiovisual/flash": ".swf",
     "code/batch": ".bat",
     "code/c": ".c",
@@ -67,6 +71,7 @@ type_to_extension = {
     "document/office/onenote": ".one",
     "document/pdf": ".pdf",
     "document/email": ".eml",
+    "executable/web/wasm": ".wasm",
     "executable/windows/pe32": ".exe",
     "executable/windows/pe64": ".exe",
     "executable/windows/dll32": ".dll",
@@ -86,6 +91,7 @@ type_to_extension = {
 magic_patterns = [
     {"al_type": "network/tnef", "regex": r"Transport Neutral Encapsulation Format"},
     {"al_type": "archive/chm", "regex": r"MS Windows HtmlHelp Data"},
+    {"al_type": "executable/web/wasm", "regex": r"WebAssembly \(wasm\) binary module"},
     {"al_type": "executable/windows/dll64", "regex": r"pe32\+[^\|]+dll[^\|]+x86\-64"},
     {"al_type": "executable/windows/pe64", "regex": r"pe32\+[^\|]+x86\-64[^\|]+windows"},
     {"al_type": "executable/windows/ia/dll64", "regex": r"pe32\+?[^\|]+dll[^\|]+Intel Itanium[^\|]+windows"},
@@ -101,7 +107,7 @@ magic_patterns = [
     {"al_type": "executable/windows/com", "regex": r"^com executable"},
     {"al_type": "executable/windows/dos", "regex": r"^8086 relocatable"},
     {"al_type": "executable/windows/coff", "regex": r"^MS Windows COFF"},
-    {"al_type": "executable/linux/elf32", "regex": r"^elf 32-bit (l|m)sb +executable"},
+    {"al_type": "executable/linux/elf32", "regex": r"^elf 32-bit (l|m)sb +(pie )?executable"},
     {"al_type": "executable/linux/elf64", "regex": r"^elf 64-bit (l|m)sb +(pie )?executable"},
     {"al_type": "executable/linux/so32", "regex": r"^elf 32-bit (l|m)sb +shared object"},
     {"al_type": "executable/linux/so64", "regex": r"^elf 64-bit (l|m)sb +shared object"},
@@ -123,6 +129,7 @@ magic_patterns = [
     {"al_type": "archive/rar", "regex": r"^rar archive data"},
     {"al_type": "archive/tar", "regex": r"^(GNU|POSIX) tar archive"},
     {"al_type": "archive/ar", "regex": r"ar archive"},
+    {"al_type": "archive/vhd", "regex": r"^Microsoft Disk Image"},
     {"al_type": "archive/xz", "regex": r"^XZ compressed data"},
     {"al_type": "archive/zip", "regex": r"^zip archive data"},
     {"al_type": "network/tcpdump", "regex": r"^(tcpdump|pcap)"},
@@ -131,6 +138,7 @@ magic_patterns = [
     {"al_type": "image/gif", "regex": r"^gif image data"},
     {"al_type": "image/jpg", "regex": r"^jpeg image data"},
     {"al_type": "image/png", "regex": r"^png image data"},
+    {"al_type": "image/tiff", "regex": r"^TIFF image data"},
     {"al_type": "image/webp", "regex": r"Web/P image"},
     {"al_type": "document/installer/windows", "regex": r"(Installation Database|Windows Installer)"},
     {"al_type": "document/office/excel", "regex": r"Microsoft[^\|]+Excel"},
@@ -217,6 +225,8 @@ trusted_mimes = {
     "text/x-shellscript": "code/shell",
     # RTF
     "text/rtf": "document/office/rtf",
+    # Troff
+    "text/troff": "text/troff",
     # Java
     "text/x-java": "code/java",
     # Batch
@@ -267,6 +277,8 @@ trusted_mimes = {
     # Image TGA
     "image/x-tga": "image/tga",
     "image/x-icb": "image/tga",
+    # Image TIFF
+    "image/tiff": "image/tiff",
     # Image Cursor
     "image/x-win-bitmap": "image/cursor",
 
@@ -328,6 +340,7 @@ trusted_mimes = {
     "application/x-arc": "archive/arc",
     "application/x-iso9660-image": "archive/iso",
     "application/x-rar": "archive/rar",
+    "application/x-virtualbox-vhd": "archive/vhd",
     "application/x-xz": "archive/xz",
     "application/vnd.ms-cab-compressed": "archive/cabinet",
 
