@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import base64
 import json
 import logging
 import re
@@ -18,8 +19,6 @@ from typing import Dict, Any, Union, TypeVar, Generic
 import elasticsearch
 import elasticsearch.helpers
 
-from elasticsearch.helpers.errors import ScanError
-
 from assemblyline import odm
 from assemblyline.common.dict_utils import recursive_update
 from assemblyline.datastore.bulk import ElasticBulkPlan
@@ -86,7 +85,7 @@ def parse_sort(sort, ret_list=True):
     """
     This function tries to do two things at once:
         - convert AL sort syntax to elastic,
-        - convert any sorts on the key _id to _id_
+        - convert any sorts on the key _id to id
     """
     if sort is None:
         return sort
@@ -95,6 +94,8 @@ def parse_sort(sort, ret_list=True):
         return [parse_sort(row, ret_list=False) for row in sort]
     elif isinstance(sort, dict):
         return {('id' if key == '_id' else key): value for key, value in sort.items()}
+    elif "," in sort:
+        return [parse_sort(row.strip(), ret_list=False) for row in sort.split(',')]
 
     parts = sort.split(' ')
     if len(parts) == 1:
@@ -130,7 +131,7 @@ class ESCollection(Generic[ModelType]):
     RETRY_NORMAL = 1
     RETRY_NONE = 0
     RETRY_INFINITY = -1
-    SCROLL_TIMEOUT = "5m"
+    KEEP_ALIVE = "5m"
     UPDATE_SET = "SET"
     UPDATE_INC = "INC"
     UPDATE_DEC = "DEC"
@@ -233,47 +234,36 @@ class ESCollection(Generic[ModelType]):
         else:
             return [self.index_name]
 
-    def scan_with_retry(self, query, sort=None, source=None, index=None, scroll="5m", size=1000, request_timeout=None):
+    def scan_with_search_after(self, query, sort=None, source=None, index=None, keep_alive=KEEP_ALIVE, size=1000,
+                               request_timeout=None):
         if index is None:
             index = self.index_name
+        if not sort:
+            sort = []
+
+        # Generate the point in time
+        pit = {'id': self.with_retries(self.datastore.client.open_point_in_time,
+                                       index=index, keep_alive=keep_alive)['id'],
+               'keep_alive': keep_alive}
+
+        # Add tie_breaker sort using _shard_doc ID
+        sort.append({"_shard_doc": "desc"})
 
         # initial search
-        resp = self.with_retries(self.datastore.client.search, index=index, query=query, scroll=scroll,
+        resp = self.with_retries(self.datastore.client.search, query=query, pit=pit,
                                  size=size, request_timeout=request_timeout, sort=sort, _source=source)
-        scroll_id = resp.get("_scroll_id")
-
         try:
-            while scroll_id and resp["hits"]["hits"]:
+            while resp["hits"]["hits"]:
+                search_after = resp['hits']['hits'][-1]['sort']
                 for hit in resp["hits"]["hits"]:
                     yield hit
 
-                # Default to 0 if the value isn't included in the response
-                shards_successful = resp["_shards"].get("successful", 0)
-                shards_skipped = resp["_shards"].get("skipped", 0)
-                shards_total = resp["_shards"].get("total", 0)
-
-                # check if we have any errors
-                if (shards_successful + shards_skipped) < shards_total:
-                    shards_message = "Scroll request has only succeeded on %d (+%d skipped) shards out of %d."
-                    raise ScanError(
-                        scroll_id,
-                        shards_message
-                        % (
-                            shards_successful,
-                            shards_skipped,
-                            shards_total,
-                        ),
-                    )
-                resp = self.with_retries(self.datastore.client.scroll, scroll_id=scroll_id, scroll=scroll)
-                scroll_id = resp.get("_scroll_id")
+                resp = self.with_retries(self.datastore.client.search, query=query, pit=pit,
+                                         size=size, request_timeout=request_timeout, sort=sort, _source=source,
+                                         search_after=search_after)
 
         finally:
-            if scroll_id:
-                resp = self.with_retries(self.datastore.client.clear_scroll, scroll_id=[scroll_id],
-                                         ignore=(404,), params={"__elastic_client_meta": (("h", "s"),)})
-                if not resp.get('succeeded', False):
-                    log.warning(f"Could not clear scroll ID {scroll_id}, there is potential "
-                                "memory leak in you Elastic cluster...")
+            self.with_retries(self.datastore.client.close_point_in_time, id=pit['id'], ignore=(404,))
 
     def with_retries(self, func, *args, raise_conflicts=False, **kwargs):
         """
@@ -842,7 +832,10 @@ class ESCollection(Generic[ModelType]):
                     log.error(f'MGet returned multiple documents for id: {row["_id"]}')
 
             if key_list and self.archive_access:
-                for row in self.scan_with_retry(query={"ids": {"values": key_list}}, index=f"{self.name}-*"):
+                result = self.with_retries(self.datastore.client.search,
+                                           index=f"{self.name}-*", query={"ids": {"values": key_list}},
+                                           size=len(key_list), collapse={"field": "id"})
+                for row in result['hits']['hits']:
                     try:
                         key_list.remove(row['_id'])
                         add_to_output(row['_source'], row['_id'])
@@ -1312,8 +1305,9 @@ class ESCollection(Generic[ModelType]):
 
             extra_fields = _strip_lists(self.model_class, extra_fields)
             if as_obj:
-                if '_index' in fields and '_index' in result:
-                    extra_fields['_index'] = result["_index"]
+                for f in fields:
+                    if f.startswith("_") and f in result:
+                        extra_fields[f] = result[f]
                 if '*' in fields:
                     fields = None
                 return self.model_class(source_data, mask=fields, docid=item_id, extra_fields=extra_fields)
@@ -1321,8 +1315,9 @@ class ESCollection(Generic[ModelType]):
                 source_data = recursive_update(source_data, extra_fields)
                 if 'id' in fields:
                     source_data['id'] = item_id
-                if '_index' in fields and '_index' in result:
-                    source_data['_index'] = result["_index"]
+                for f in fields:
+                    if f.startswith("_") and f in result:
+                        source_data[f] = result[f]
                 return source_data
 
         if isinstance(fields, str):
@@ -1344,10 +1339,30 @@ class ESCollection(Generic[ModelType]):
         if args is None:
             args = []
 
+        # Initialize values
+        pit = None
+        search_after = None
         params = {}
+
+        # If there is deep_paging
         if deep_paging_id is not None:
-            params = {'scroll': self.SCROLL_TIMEOUT}
-        elif track_total_hits:
+            # If we have a properly formatted deep paging ID
+            if "," in deep_paging_id:
+                # Parse deep paging ID for pit_id and search_after
+                pit_id, search_after_b64 = deep_paging_id.split(',')
+                pit = {'id': pit_id, 'keep_alive': self.KEEP_ALIVE}
+                search_after = json.loads(base64.b64decode(search_after_b64))
+
+            else:
+                # Create a new deep paging ID
+                pit = {'id': self.with_retries(self.datastore.client.open_point_in_time,
+                                               index=index, keep_alive=self.KEEP_ALIVE)['id'],
+                       'keep_alive': self.KEEP_ALIVE}
+
+            # Completely disable hit tracking to speed things up
+            params = {'track_total_hits': False}
+
+        if track_total_hits:
             params['track_total_hits'] = track_total_hits
 
         parsed_values = deepcopy(self.DEFAULT_SEARCH_VALUES)
@@ -1360,6 +1375,8 @@ class ESCollection(Generic[ModelType]):
 
             parsed_values[key] = value
 
+        field_list = parsed_values['field_list'] or list(self.stored_fields.keys())
+
         # This is our minimal query, the following sections will fill it out
         # with whatever extra options the search has been given.
         query_body = {
@@ -1376,9 +1393,12 @@ class ESCollection(Generic[ModelType]):
             'from_': parsed_values['start'],
             'size': parsed_values['rows'],
             'sort': parse_sort(parsed_values['sort']),
-            "_source": parsed_values['field_list'] or list(self.stored_fields.keys())
+            "_source": field_list
         }
 
+        if "_seq_no" in field_list or "_primary_term" in field_list:
+            query_body["seq_no_primary_term"] = True
+
         if parsed_values['script_fields']:
             fields = {}
             for (f_name, f_script) in parsed_values['script_fields']:
@@ -1470,9 +1490,13 @@ class ESCollection(Generic[ModelType]):
             }
 
         try:
-            if deep_paging_id is not None and not deep_paging_id == "*":
-                # Get the next page
-                result = self.with_retries(self.datastore.client.scroll, scroll_id=deep_paging_id, params=params)
+            # If we are using a Point in Time
+            if pit:
+                # Add a tie breaker sort field
+                query_body['sort'].append({"_shard_doc": "desc"})
+                # Get the next result page for PIT + Search After
+                result = self.with_retries(self.datastore.client.search, params=params, pit=pit,
+                                           search_after=search_after, **query_body)
             else:
                 # Run the query
                 result = self.with_retries(self.datastore.client.search, index=index,
@@ -1491,7 +1515,8 @@ class ESCollection(Generic[ModelType]):
             raise SearchException(err_msg)
 
         except Exception as error:
-            raise SearchException("collection: %s, query: %s, error: %s" % (self.name, query_body, str(error)))
+            raise SearchException("collection: %s, query: %s, error: %s" % (
+                self.name, query_body, str(error))).with_traceback(error.__traceback__)
 
     def search(self, query, offset=0, rows=None, sort=None,
                fl=None, timeout=None, filters=None, access_control=None,
@@ -1571,23 +1596,26 @@ class ESCollection(Generic[ModelType]):
         ret_data = {
             "offset": int(offset),
             "rows": int(rows),
-            "total": int(result['hits']['total']['value']),
+            "total": result['hits'].get('total', {}).get('value', None),
             "items": [self._format_output(doc, field_list, as_obj=as_obj) for doc in result['hits']['hits']]
         }
 
-        new_deep_paging_id = result.get("_scroll_id", None)
+        # If we where tracking total, change it into an INT
+        if ret_data['total'] is not None:
+            ret_data['total'] = int(ret_data['total'])
 
-        # Check if the scroll is finished and close it
-        if deep_paging_id is not None and new_deep_paging_id is None:
-            self.with_retries(self.datastore.client.clear_scroll, scroll_id=[deep_paging_id], ignore=(404,))
+        # Get the currently used Point in Time if it exists
+        pit_id = result.get("pit_id", None)
 
-        # Check if we can tell from inspection that we have finished the scroll
-        if new_deep_paging_id is not None and len(ret_data["items"]) < ret_data["rows"]:
-            self.with_retries(self.datastore.client.clear_scroll, scroll_id=[new_deep_paging_id], ignore=(404,))
-            new_deep_paging_id = None
-
-        if new_deep_paging_id is not None:
-            ret_data['next_deep_paging_id'] = new_deep_paging_id
+        # Check if deep paging is over
+        if pit_id is not None and len(ret_data["items"]) < ret_data["rows"]:
+            # Close the Point in Time
+            self.with_retries(self.datastore.client.close_point_in_time, id=pit_id, ignore=(404,))
+        elif pit_id is not None and len(ret_data["items"]) > 0:
+            # We have a PIT ID and we don't seem to have finished looping throught the data
+            # create the next deep paging id for the user to use.
+            search_after = base64.b64encode(json.dumps(result['hits']['hits'][-1]['sort']).encode()).decode()
+            ret_data['next_deep_paging_id'] = f"{pit_id},{search_after}"
 
         return ret_data
 
@@ -1605,13 +1633,13 @@ class ESCollection(Generic[ModelType]):
         >>>     fl[x]: value
         >>> }
 
-        :param use_archive: Query also the archive
-        :param as_obj: Return objects instead of dictionaries
         :param query: lucene query to search for
         :param fl: list of fields to return from the search
         :param filters: additional queries to run on the original query to reduce the scope
         :param access_control: access control parameters to run the query with
-        :param buffer_size: number of items to buffer with each search call
+        :param item_buffer_size: number of items to buffer with each search call
+        :param as_obj: Return objects instead of dictionaries
+        :param use_archive: Query also the archive
         :return: a generator of dictionary of field list results
         """
         if item_buffer_size > 2000 or item_buffer_size < 50:
@@ -1646,8 +1674,8 @@ class ESCollection(Generic[ModelType]):
         sort = parse_sort(self.datastore.DEFAULT_SORT)
         source = fl or list(self.stored_fields.keys())
 
-        for value in self.scan_with_retry(query=query_expression, sort=sort, source=source,
-                                          index=index, size=item_buffer_size):
+        for value in self.scan_with_search_after(query=query_expression, sort=sort, source=source,
+                                                 index=index, size=item_buffer_size):
             # Unpack the results, ensure the id is always set
             yield self._format_output(value, fl, as_obj=as_obj)
 
@@ -1669,7 +1697,7 @@ class ESCollection(Generic[ModelType]):
                 else:
                     yield value
 
-    def _validate_steps_count(self, start, end, gap):
+    def _validate_steps_count(self, start, end, gap) -> type:
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
 
@@ -1914,43 +1942,45 @@ class ESCollection(Generic[ModelType]):
         return collection_data
 
     def _ilm_policy_exists(self):
-        conn = self.datastore.client.transport.get_connection()
-        pol_req = conn.session.get(f"{conn.base_url}/_ilm/policy/{self.name}_policy")
-        return pol_req.ok
+        try:
+            self.datastore.client.ilm.get_lifecycle(name=f"{self.name}_policy")
+            return True
+        except elasticsearch.NotFoundError:
+            return False
 
     def _delete_ilm_policy(self):
-        conn = self.datastore.client.transport.get_connection()
-        pol_req = conn.session.delete(f"{conn.base_url}/_ilm/policy/{self.name}_policy")
-        return pol_req.ok
+        try:
+            self.datastore.client.ilm.delete_lifecycle(name=f"{self.name}_policy")
+            return True
+        except elasticsearch.ApiError:
+            return False
 
     def _create_ilm_policy(self):
         data_base = {
-            "policy": {
-                "phases": {
-                    "hot": {
-                        "min_age": "0ms",
-                        "actions": {
-                            "set_priority": {
-                                "priority": 100
-                            },
-                            "rollover": {
-                                "max_age": f"{self.ilm_config['warm']}{self.ilm_config['unit']}"
-                            }
+            "phases": {
+                "hot": {
+                    "min_age": "0ms",
+                    "actions": {
+                        "set_priority": {
+                            "priority": 100
+                        },
+                        "rollover": {
+                            "max_age": f"{self.ilm_config['warm']}{self.ilm_config['unit']}"
                         }
-                    },
-                    "warm": {
-                        "actions": {
-                            "set_priority": {
-                                "priority": 50
-                            }
+                    }
+                },
+                "warm": {
+                    "actions": {
+                        "set_priority": {
+                            "priority": 50
                         }
-                    },
-                    "cold": {
-                        "min_age": f"{self.ilm_config['cold']}{self.ilm_config['unit']}",
-                        "actions": {
-                            "set_priority": {
-                                "priority": 20
-                            }
+                    }
+                },
+                "cold": {
+                    "min_age": f"{self.ilm_config['cold']}{self.ilm_config['unit']}",
+                    "actions": {
+                        "set_priority": {
+                            "priority": 20
                         }
                     }
                 }
@@ -1958,18 +1988,16 @@ class ESCollection(Generic[ModelType]):
         }
 
         if self.ilm_config['delete']:
-            data_base['policy']['phases']['delete'] = {
+            data_base['phases']['delete'] = {
                 "min_age": f"{self.ilm_config['delete']}{self.ilm_config['unit']}",
                 "actions": {
                     "delete": {}
                 }
             }
 
-        conn = self.datastore.client.transport.get_connection()
-        pol_req = conn.session.put(f"{conn.base_url}/_ilm/policy/{self.name}_policy",
-                                   headers={"Content-Type": "application/json"},
-                                   data=json.dumps(data_base))
-        if not pol_req.ok:
+        try:
+            self.datastore.client.ilm.put_lifecycle(name=f"{self.name}_policy", policy=data_base)
+        except elasticsearch.ApiError:
             raise ILMException(f"ERROR: Failed to create ILM policy: {self.name}_policy")
 
     def _get_index_settings(self) -> dict:
