@@ -833,6 +833,8 @@ class Services(odm.Model):
     safelist = odm.Compound(ServiceSafelist)
     registries = odm.Optional(odm.List(odm.Compound(ServiceRegistry)),
                               description="Global set of registries for services")
+    service_account = odm.optional(odm.keyword(description="Service account to use for pods in kubernetes "
+                                                           "where the service does not have one configured."))
 
 
 DEFAULT_SERVICES = {
@@ -956,6 +958,7 @@ class UI(odm.Model):
     read_only: bool = odm.Boolean(description="Turn on read only mode in the UI")
     read_only_offset: str = odm.Keyword(
         default="", description="Offset of the read only mode for all paging and searches")
+    rss_feeds: List[str] = odm.List(odm.Keyword(), default=[], description="List of RSS feeds to display on the UI")
     secret_key: str = odm.Keyword(description="Flask secret key to store cookies, etc.")
     session_duration: int = odm.Integer(description="Duration of the user session before the user has to login again")
     statistics: Statistics = odm.Compound(Statistics, default=DEFAULT_STATISTICS,
@@ -968,6 +971,7 @@ class UI(odm.Model):
                                                           description="Headers used by the url_download method")
     url_submission_proxies: Dict[str, str] = odm.Optional(odm.Mapping(odm.Keyword()),
                                                           description="Proxy used by the url_download method")
+    url_submission_timeout: int = odm.Integer(default=15, description="Request timeout for fetching URLs")
     validate_session_ip: bool = \
         odm.Boolean(description="Validate if the session IP matches the IP the session was created from")
     validate_session_useragent: bool = \
@@ -993,6 +997,10 @@ DEFAULT_UI = {
     "ingest_max_priority": 250,
     "read_only": False,
     "read_only_offset": "",
+    "rss_feeds": [
+        "https://alpytest.blob.core.windows.net/pytest/stable.json",
+        "https://alpytest.blob.core.windows.net/pytest/services.json"
+    ],
     "secret_key": "This is the default flask secret key... you should change this!",
     "session_duration": 3600,
     "statistics": DEFAULT_STATISTICS,
