@@ -1,6 +1,11 @@
+import typing
+from collections import defaultdict
+
 from assemblyline import odm
 from assemblyline.common import forge
 from assemblyline.common.caching import generate_conf_key
+from assemblyline.common.dict_utils import flatten
+from assemblyline.common.tagging import tag_dict_to_list
 from assemblyline.odm.models.tagging import Tagging
 
 
@@ -77,6 +82,9 @@ class File(odm.Model):
     classification = odm.Classification(description="Classification of the file")
     is_section_image = odm.Boolean(default=False,
                                    description="Is this an image used in an Image Result Section?")
+    allow_dynamic_recursion = odm.Boolean(
+        default=False,
+        description="Allow file to be analysed during Dynamic Analysis even if Dynamic Recursion Prevention is enabled.")
 
 
 @odm.model(index=True, store=True, description="Response Body of Result")
@@ -128,7 +136,19 @@ class Result(odm.Model):
 
         return '.'.join(key_list)
 
-    def is_empty(self):
+    def scored_tag_dict(self) -> dict[str, dict[str, typing.Any]]:
+        tags = defaultdict(lambda: {'score': 0})
+        # Save the tags and their score
+        for section in self.result.sections:
+            tag_list = tag_dict_to_list(flatten(section.tags.as_primitives()))
+            for tag in tag_list:
+                key = f"{tag['type']}:{tag['value']}"
+                tags[key].update(tag)
+                tags[key]['score'] += section.heuristic.score if section.heuristic else 0
+
+        return tags
+
+    def is_empty(self) -> bool:
         if len(self.response.extracted) == 0 and \
                 len(self.response.supplementary) == 0 and \
                 len(self.result.sections) == 0 and \
