@@ -57,7 +57,7 @@ def retry_call(func, *args, **kw):
 
             return ret_val
 
-        except redis.ConnectionError as ce:
+        except (redis.ConnectionError, ConnectionResetError) as ce:
             log.warning(f'No connection to Redis, reconnecting... [{ce}]')
             time.sleep(2 ** exponent)
             exponent = exponent + 1 if exponent < maximum else exponent
@@ -69,15 +69,15 @@ def get_client(host, port, private):
         return host
 
     if not host or not port:
-        config = forge.get_config(static=True)
+        config = forge.get_config()
 
         host = host or config.core.redis.nonpersistent.host
         port = int(port or config.core.redis.nonpersistent.port)
 
     if private:
-        return redis.StrictRedis(host=host, port=port)
+        return redis.StrictRedis(host=host, port=port, socket_keepalive=True)
     else:
-        return redis.StrictRedis(connection_pool=get_pool(host, port))
+        return redis.StrictRedis(connection_pool=get_pool(host, port), socket_keepalive=True)
 
 
 def get_pool(host, port):
